==> masm-ses-0.0.1.tgz: package/main.js <==
var crypto    = require("crypto");
var https     = require("https");
var templates = require("masm-template");

/*
 *  Use SES to send emails.
 */

const ses_endpoint = "email.us-east-1.amazonaws.com";

module.exports = function (email, aws_access_key_id, aws_secret_key) {
    const defaultEmailSettings = {
        from: email
    };

    return {
        send : sendEmailUsingSES
    };

    // This procedure returns the signature that should be used to sign the SES request.
    // String -> String
    function calculate_signature(date) {
        var h = crypto.createHmac("sha256", aws_secret_key);
        h.update(date);
        return "AWS3-HTTPS AWSAccessKeyId=" + aws_access_key_id + ", Algorithm=HMACSHA256, Signature=" + h.digest("base64");
    }

    const days_vec = ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"];
    const months_vec = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"];

    // Given a date, this procedure returns the a string representation using the format in rfc2616.
    // Date -> String
    function date_to_rfc2616_string(date) {
        return days_vec[date.getUTCDay()] + ", " +
            date.getUTCDate() + " " + months_vec[date.getUTCMonth()] + " " + date.getUTCFullYear() + " " +
            at_least_two_chars(date.getUTCHours()) + ":" + at_least_two_chars(date.getUTCMinutes()) + ":" + at_least_two_chars(date.getUTCSeconds()) + " +0000";
    }

    function at_least_two_chars(num) {
        return num<0 ? "00" : (num<10 ? "0"+num : num.toString());
    }

    // This procedure returns the URL's path that should be used to send an email.  The argument
    // should be an object with the configuration data to send the email.
    // String -> String
    function path_string(obj) {
        return "/?Action=SendEmail&Source=" + encodeURIComponent(obj.from) +
            "&Destination.ToAddresses.member.1=" + encodeURIComponent(obj.to) +
            "&Message.Subject.Data=" + encodeURIComponent(obj.subject) +
            "&Message.Body.Text.Data=" + encodeURIComponent(obj.body);
    }

    // This procedure send an email using SES. The argument should be the configuration data to send the email.
    // If the arguments does not have a |body| property, then if is necessary to load the Mustache template.
    // Otherwise, it really sends the email using |reallySendEmailUsingSES|.
    // Object -> undefined
    function sendEmailUsingSES(obj, callback) {
        console.log("Sending email to: " + JSON.stringify(obj));
        if (obj.body) {
            return reallySendEmailUsingSES(obj, callback);
        } else {
            return templates.afterLoadingTemplate(obj, function (obj) {
                return reallySendEmailUsingSES(obj, callback);
            });
        }
    }

    // This procedure really sends the email.  The |obj| argument is the email configuration
    // data.  The |error_proc| argument should be a procedure that is invoked when an error occurs.
    // Object -> (Error -> undefined) -> undefined
    function reallySendEmailUsingSES(obj, callback) {
        if (!obj.date) {
            obj.date = new Date();
        }
        if (!obj.from) {
            obj.from = defaultEmailSettings.from;
        }
        var date_str = date_to_rfc2616_string(obj.date || new Date());
        https.get({
            host: ses_endpoint,
            port: 443,
            path: path_string(obj),
            headers: {
                "Host": ses_endpoint,
                "Date": date_str,
                "X-Amzn-Authorization": calculate_signature(date_str)
            }
        }, function(res) {
            if (res.statusCode == 200) {
                callback(false);
            } else {
                var status = res.statusCode;
                var headers = res.headers;
                var body = [];
                res.setEncoding("utf8");
                res.on("data", function(chunk) {
                    body.push(chunk);
                });
                res.on("end", function() {
                    callback({
                        error:"mailer failed",
                        message: [
                            "SES error: ",
                            status,
                            "\nheaders: ",
                            JSON.stringify(headers),
                            "\nbody: ",
                            body.join("")
                        ].join("")
                    });
                });
            }
        }).on('error', function () {
            callback({error: "internal server error", message: "While sending email, got error: " + e.message});
        });
    }
};

==> masm-ses-0.0.2.tgz: package/main.js <==
var crypto    = require("crypto");
var https     = require("https");
var templates = require("masm-template");

/*
 *  Use SES to send emails.
 */

const ses_endpoint = "email.us-east-1.amazonaws.com";

const days_vec = ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"];
const months_vec = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"];

module.exports = function (email, aws_access_key_id, aws_secret_key) {
    const defaultEmailSettings = {
        from: email
    };

    return {
        send : sendEmailUsingSES
    };

    // This procedure returns the signature that should be used to sign the SES request.
    // String -> String
    function calculate_signature(date) {
        var h = crypto.createHmac("sha256", aws_secret_key);
        h.update(date);
        return "AWS3-HTTPS AWSAccessKeyId=" + aws_access_key_id + ", Algorithm=HMACSHA256, Signature=" + h.digest("base64");
    }

    // Given a date, this procedure returns the a string representation using the format in rfc2616.
    // Date -> String
    function date_to_rfc2616_string(date) {
        return days_vec[date.getUTCDay()] + ", " +
            date.getUTCDate() + " " + months_vec[date.getUTCMonth()] + " " + date.getUTCFullYear() + " " +
            at_least_two_chars(date.getUTCHours()) + ":" + at_least_two_chars(date.getUTCMinutes()) + ":" + at_least_two_chars(date.getUTCSeconds()) + " +0000";
    }

    function at_least_two_chars(num) {
        return num<0 ? "00" : (num<10 ? "0"+num : num.toString());
    }

    // This procedure returns the URL's path that should be used to send an email.  The argument
    // should be an object with the configuration data to send the email.
    // String -> String
    function path_string(obj) {
        return "/?Action=SendEmail&Source=" + encodeURIComponent(obj.from) +
            "&Destination.ToAddresses.member.1=" + encodeURIComponent(obj.to) +
            "&Message.Subject.Data=" + encodeURIComponent(obj.subject) +
            "&Message.Body.Text.Data=" + encodeURIComponent(obj.body);
    }

    // This procedure send an email using SES. The argument should be the configuration data to send the email.
    // If the arguments does not have a |body| property, then if is necessary to load the Mustache template.
    // Otherwise, it really sends the email using |reallySendEmailUsingSES|.
    // Object -> undefined
    function sendEmailUsingSES(obj, callback) {
        console.log("Sending email to: " + JSON.stringify(obj));
        if (obj.body) {
            return reallySendEmailUsingSES(obj, callback);
        } else {
            return templates.afterLoadingTemplate(obj, function (obj) {
                return reallySendEmailUsingSES(obj, callback);
            });
        }
    }

    // This procedure really sends the email.  The |obj| argument is the email configuration
    // data.  The |error_proc| argument should be a procedure that is invoked when an error occurs.
    // Object -> (Error -> undefined) -> undefined
    function reallySendEmailUsingSES(obj, callback) {
        if (!obj.date) {
            obj.date = new Date();
        }
        if (!obj.from) {
            obj.from = defaultEmailSettings.from;
        }
        var date_str = date_to_rfc2616_string(obj.date);
        https.get({
            host: ses_endpoint,
            port: 443,
            path: path_string(obj),
            headers: {
                "Host": ses_endpoint,
                "Date": date_str,
                "X-Amzn-Authorization": calculate_signature(date_str)
            }
        }, function(res) {
            if (res.statusCode == 200) {
                callback(false);
            } else {
                var status = res.statusCode;
                var headers = res.headers;
                var body = [];
                res.setEncoding("utf8");
                res.on("data", function(chunk) {
                    body.push(chunk);
                });
                res.on("end", function() {
                    callback({
                        error:"mailer failed",
                        message: [
                            "SES error: ",
                            status,
                            "\nheaders: ",
                            JSON.stringify(headers),
                            "\nbody: ",
                            body.join("")
                        ].join("")
                    });
                });
            }
        }).on('error', function () {
            callback({error: "internal server error", message: "While sending email, got error: " + e.message});
        });
    }
};
